@@ -96,15 +96,23 @@ struct ldb_dn *sysdb_custom_subtree_dn(struct sysdb_ctx *ctx, void *memctx,
     errno_t ret;
     char *clean_subtree;
     struct ldb_dn *dn = NULL;
+    TALLOC_CTX *tmp_ctx;
 
-    ret = sysdb_dn_sanitize(NULL, subtree_name, &clean_subtree);
+    tmp_ctx = talloc_new(memctx);
+    if (!tmp_ctx) return NULL;
+
+    ret = sysdb_dn_sanitize(tmp_ctx, subtree_name, &clean_subtree);
     if (ret != EOK) {
+        talloc_free(tmp_ctx);
         return NULL;
     }
 
-    dn = ldb_dn_new_fmt(memctx, ctx->ldb, SYSDB_TMPL_CUSTOM_SUBTREE,
+    dn = ldb_dn_new_fmt(tmp_ctx, ctx->ldb, SYSDB_TMPL_CUSTOM_SUBTREE,
                         clean_subtree, domain);
-    talloc_free(clean_subtree);
+    if (dn) {
+        talloc_steal(memctx, dn);
+    }
+    talloc_free(tmp_ctx);
 
     return dn;
 }
@@ -809,7 +817,7 @@ static int sysdb_upgrade_01(TALLOC_CTX *mem_ctx,
     char *domain;
     int ret, i, j;
 
-    basedn = ldb_dn_new(mem_ctx, ldb, "cn=sysdb");
+    basedn = ldb_dn_new(mem_ctx, ldb, SYSDB_BASE);
     if (!basedn) {
         ret = EIO;
         goto done;
@@ -903,7 +911,7 @@ static int sysdb_upgrade_01(TALLOC_CTX *mem_ctx,
         ret = ENOMEM;
         goto done;
     }
-    msg->dn = ldb_dn_new(mem_ctx, ldb, "cn=sysdb");
+    msg->dn = ldb_dn_new(mem_ctx, ldb, SYSDB_BASE);
     if (!msg->dn) {
         ret = ENOMEM;
         goto done;
@@ -979,7 +987,7 @@ static int sysdb_check_upgrade_02(TALLOC_CTX *mem_ctx,
         return ret;
     }
 
-    verdn = ldb_dn_new(tmp_ctx, ldb, "cn=sysdb");
+    verdn = ldb_dn_new(tmp_ctx, ldb, SYSDB_BASE);
     if (!verdn) {
         ret = EIO;
         goto exit;
@@ -1222,7 +1230,7 @@ static int sysdb_check_upgrade_02(TALLOC_CTX *mem_ctx,
         ret = ENOMEM;
         goto done;
     }
-    msg->dn = ldb_dn_new(tmp_ctx, ldb, "cn=sysdb");
+    msg->dn = ldb_dn_new(tmp_ctx, ldb, SYSDB_BASE);
     if (!msg->dn) {
         ret = ENOMEM;
         goto done;
@@ -1322,7 +1330,7 @@ static int sysdb_upgrade_03(struct sysdb_ctx *ctx, const char **ver)
         ret = ENOMEM;
         goto done;
     }
-    msg->dn = ldb_dn_new(tmp_ctx, ctx->ldb, "cn=sysdb");
+    msg->dn = ldb_dn_new(tmp_ctx, ctx->ldb, SYSDB_BASE);
     if (!msg->dn) {
         ret = ENOMEM;
         goto done;
@@ -1435,7 +1443,7 @@ static int sysdb_upgrade_04(struct sysdb_ctx *ctx, const char **ver)
         ret = ENOMEM;
         goto done;
     }
-    msg->dn = ldb_dn_new(tmp_ctx, ctx->ldb, "cn=sysdb");
+    msg->dn = ldb_dn_new(tmp_ctx, ctx->ldb, SYSDB_BASE);
     if (!msg->dn) {
         ret = ENOMEM;
         goto done;
@@ -1686,7 +1694,7 @@ static int sysdb_upgrade_07(struct sysdb_ctx *ctx, const char **ver)
     int ret;
     struct ldb_message *msg;
 
-    tmp_ctx = talloc_new(ctx);
+    tmp_ctx = talloc_new(NULL);
     if (!tmp_ctx) {
         return ENOMEM;
     }
@@ -1809,22 +1817,23 @@ static int sysdb_domain_init_internal(TALLOC_CTX *mem_ctx,
                             domain->name, db_path,
                             &ctx->ldb_file);
     if (ret != EOK) {
-        return ret;
+        goto done;
     }
     DEBUG(5, ("DB File for %s: %s\n", domain->name, ctx->ldb_file));
 
     ret = sysdb_ldb_connect(ctx, ctx->ldb_file, &ctx->ldb);
     if (ret != EOK) {
         DEBUG(1, ("sysdb_ldb_connect failed.\n"));
-        return ret;
+        goto done;
     }
 
     tmp_ctx = talloc_new(ctx);
     if (!tmp_ctx) {
-        return ENOMEM;
+        ret = ENOMEM;
+        goto done;
     }
 
-    verdn = ldb_dn_new(tmp_ctx, ctx->ldb, "cn=sysdb");
+    verdn = ldb_dn_new(tmp_ctx, ctx->ldb, SYSDB_BASE);
     if (!verdn) {
         ret = EIO;
         goto done;
@@ -1934,7 +1943,7 @@ static int sysdb_domain_init_internal(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-    /* cn=sysdb does not exists, means db is empty, populate */
+    /* SYSDB_BASE does not exists, means db is empty, populate */
 
     base_ldif = SYSDB_BASE_LDIF;
     while ((ldif = ldb_ldif_read_string(ctx->ldb, &base_ldif))) {
@@ -2031,11 +2040,23 @@ static int sysdb_domain_init_internal(TALLOC_CTX *mem_ctx,
     }
     talloc_zfree(msg);
 
-    ret = EOK;
+    /* The cache has been newly created.
+     * We need to reopen the LDB to ensure that
+     * all of the special values take effect
+     * (such as enabling the memberOf plugin and
+     * the various indexes).
+     */
+    talloc_zfree(ctx->ldb);
+    ret = sysdb_ldb_connect(ctx, ctx->ldb_file, &ctx->ldb);
+    if (ret != EOK) {
+        DEBUG(1, ("sysdb_ldb_connect failed.\n"));
+    }
 
 done:
     if (ret == EOK) {
         *_ctx = ctx;
+    } else {
+        talloc_free(ctx);
     }
     talloc_free(tmp_ctx);
     return ret;
@@ -2125,6 +2146,47 @@ int sysdb_domain_init(TALLOC_CTX *mem_ctx,
                                       db_path, false, _ctx);
 }
 
+int sysdb_list_init(TALLOC_CTX *mem_ctx,
+                    const char *path,
+                    struct sysdb_ctx *ctx,
+                    struct sysdb_ctx_list **_list)
+{
+    struct sysdb_ctx_list *list;
+    int ret;
+
+    list = talloc_zero(mem_ctx, struct sysdb_ctx_list);
+    if (!list) {
+        DEBUG(1, ("talloc_zero failed\n"));
+        return ENOMEM;
+    }
+
+    list->db_path = talloc_strdup(list, path);
+    if (!list->db_path) {
+        DEBUG(1, ("talloc_strdup failed\n"));
+        ret = ENOMEM;
+        goto fail;
+    }
+
+    if (ctx) {
+        list->num_dbs = 1;
+        list->dbs = talloc_array(list, struct sysdb_ctx *, list->num_dbs);
+        if (!list->dbs) {
+            DEBUG(1, ("talloc_array failed\n"));
+            ret = ENOMEM;
+            goto fail;
+        }
+
+        list->dbs[0] = talloc_steal(list, ctx);
+    }
+
+    *_list = list;
+    return EOK;
+
+fail:
+    talloc_free(list);
+    return ret;
+}
+
 int sysdb_get_ctx_from_list(struct sysdb_ctx_list *ctx_list,
                             struct sss_domain_info *domain,
                             struct sysdb_ctx **ctx)
