@@ -81,6 +81,7 @@
 #define SYSDB_INITGR_EXPIRE "initgrExpireTimestamp"
 
 #define SYSDB_AUTHORIZED_SERVICE "authorizedService"
+#define SYSDB_AUTHORIZED_HOST "authorizedHost"
 
 #define SYSDB_NETGROUP_TRIPLE "netgroupTriple"
 #define SYSDB_ORIG_NETGROUP_MEMBER "originalMemberNisNetgroup"
@@ -170,9 +171,15 @@
 #define SYSDB_MOD_REP LDB_FLAG_MOD_REPLACE
 
 struct confdb_ctx;
-struct sysdb_ctx_list;
 struct sysdb_ctx;
 
+struct sysdb_ctx_list {
+    struct sysdb_ctx **dbs;
+    size_t num_dbs;
+
+    char *db_path;
+};
+
 struct sysdb_attrs {
     int num;
     struct ldb_message_element *a;
@@ -289,6 +296,11 @@ int sysdb_domain_init(TALLOC_CTX *mem_ctx,
                       const char *db_path,
                       struct sysdb_ctx **_ctx);
 
+int sysdb_list_init(TALLOC_CTX *mem_ctx,
+                    const char *path,
+                    struct sysdb_ctx *ctx,
+                    struct sysdb_ctx_list **_list);
+
 int sysdb_get_ctx_from_list(struct sysdb_ctx_list *ctx_list,
                             struct sss_domain_info *domain,
                             struct sysdb_ctx **_ctx);
@@ -491,7 +503,8 @@ int sysdb_add_user(TALLOC_CTX *mem_ctx,
 
 int sysdb_add_fake_user(struct sysdb_ctx *ctx,
                         struct sss_domain_info *domain,
-                        const char *name);
+                        const char *name,
+                        const char *original_dn);
 
 /* Add group (only basic attrs and w/o checks) */
 int sysdb_add_basic_group(TALLOC_CTX *mem_ctx,
@@ -709,6 +722,14 @@ int sysdb_delete_group(TALLOC_CTX *mem_ctx,
                        struct sss_domain_info *domain,
                        const char *name, gid_t gid);
 
+int sysdb_search_netgroups(TALLOC_CTX *mem_ctx,
+                           struct sysdb_ctx *sysdb,
+                           struct sss_domain_info *domain,
+                           const char *sub_filter,
+                           const char **attrs,
+                           size_t *msgs_count,
+                           struct ldb_message ***msgs);
+
 int sysdb_delete_netgroup(struct sysdb_ctx *sysdb,
                           struct sss_domain_info *domain,
                           const char *name);
