@@ -486,6 +486,10 @@ int sysdb_set_user_attr(TALLOC_CTX *mem_ctx,
 {
     struct ldb_dn *dn;
 
+    if (!domain) {
+        domain = ctx->domain;
+    }
+
     dn = sysdb_user_dn(ctx, mem_ctx, domain->name, name);
     if (!dn) {
         return ENOMEM;
@@ -531,6 +535,10 @@ int sysdb_set_netgroup_attr(struct sysdb_ctx *ctx,
         return ENOMEM;
     }
 
+    if (domain == NULL) {
+        domain = ctx->domain;
+    }
+
     dn = sysdb_netgroup_dn(ctx, tmp_ctx, domain->name, name);
     if (!dn) {
         ret = ENOMEM;
@@ -926,7 +934,8 @@ done:
 
 int sysdb_add_fake_user(struct sysdb_ctx *ctx,
                         struct sss_domain_info *domain,
-                        const char *name)
+                        const char *name,
+                        const char *original_dn)
 {
     TALLOC_CTX *tmpctx;
     struct ldb_message *msg;
@@ -975,6 +984,12 @@ int sysdb_add_fake_user(struct sysdb_ctx *ctx,
                     (unsigned long) now-1);
     if (ret) goto done;
 
+    if (original_dn) {
+        ret = add_string(msg, LDB_FLAG_MOD_ADD,
+                         SYSDB_ORIG_DN, original_dn);
+        if (ret) goto done;
+    }
+
     ret = ldb_add(ctx->ldb, msg);
     ret = sysdb_error_to_errno(ret);
 
@@ -1087,8 +1102,13 @@ int sysdb_add_group(TALLOC_CTX *mem_ctx,
         ret = sysdb_search_group_by_gid(tmpctx, ctx,
                                         domain, gid, NULL, &msg);
         if (ret != ENOENT) {
-            if (ret == EOK) ret = EEXIST;
-            goto done;
+            if (ret == EOK) {
+                ret = sysdb_delete_group(tmpctx, ctx, domain, NULL, gid);
+            }
+
+            if (ret != EOK) {
+                goto done;
+            }
         }
     }
 
@@ -1389,6 +1409,18 @@ int sysdb_store_user(TALLOC_CTX *mem_ctx,
         return ENOMEM;
     }
 
+    if (!domain) {
+        domain = ctx->domain;
+    }
+
+    if (!attrs) {
+        attrs = sysdb_new_attrs(tmpctx);
+        if (!attrs) {
+            ret = ENOMEM;
+            goto done;
+        }
+    }
+
     if (pwd && (domain->legacy_passwords || !*pwd)) {
         ret = sysdb_attrs_add_string(attrs, SYSDB_PWD, pwd);
         if (ret) goto done;
@@ -1413,14 +1445,6 @@ int sysdb_store_user(TALLOC_CTX *mem_ctx,
     }
 
     /* the user exists, let's just replace attributes when set */
-    if (!attrs) {
-        attrs = sysdb_new_attrs(tmpctx);
-        if (!attrs) {
-            ret = ENOMEM;
-            goto done;
-        }
-    }
-
     if (uid) {
         ret = sysdb_attrs_add_uint32(attrs, SYSDB_UIDNUM, uid);
         if (ret) goto done;
@@ -1522,6 +1546,10 @@ int sysdb_store_group(TALLOC_CTX *mem_ctx,
         return ENOMEM;
     }
 
+    if (!domain) {
+        domain = ctx->domain;
+    }
+
     ret = sysdb_search_group_by_name(tmpctx, ctx,
                                      domain, name, src_attrs, &msg);
     if (ret && ret != ENOENT) {
@@ -1531,6 +1559,14 @@ int sysdb_store_group(TALLOC_CTX *mem_ctx,
         new_group = true;
     }
 
+    if (!attrs) {
+        attrs = sysdb_new_attrs(tmpctx);
+        if (!attrs) {
+            ret = ENOMEM;
+            goto done;
+        }
+    }
+
     /* FIXME: use the remote modification timestamp to know if the
      * group needs any update */
 
@@ -1542,15 +1578,6 @@ int sysdb_store_group(TALLOC_CTX *mem_ctx,
     }
 
     /* the group exists, let's just replace attributes when set */
-
-    if (!attrs) {
-        attrs = sysdb_new_attrs(tmpctx);
-        if (!attrs) {
-            ret = ENOMEM;
-            goto done;
-        }
-    }
-
     if (gid) {
         ret = sysdb_attrs_add_uint32(attrs, SYSDB_GIDNUM, gid);
         if (ret) goto done;
@@ -2094,6 +2121,10 @@ int sysdb_search_users(TALLOC_CTX *mem_ctx,
         return ENOMEM;
     }
 
+    if (!domain) {
+        domain = sysdb->domain;
+    }
+
     basedn = ldb_dn_new_fmt(tmpctx, sysdb->ldb,
                             SYSDB_TMPL_USER_BASE, domain->name);
     if (!basedn) {
@@ -2208,6 +2239,10 @@ int sysdb_search_groups(TALLOC_CTX *mem_ctx,
         return ENOMEM;
     }
 
+    if (!domain) {
+        domain = sysdb->domain;
+    }
+
     basedn = ldb_dn_new_fmt(tmpctx, sysdb->ldb,
                             SYSDB_TMPL_GROUP_BASE, domain->name);
     if (!basedn) {
@@ -2301,6 +2336,63 @@ fail:
     return ret;
 }
 
+/* =Search-Netgroups-with-Custom-Filter===================================== */
+
+int sysdb_search_netgroups(TALLOC_CTX *mem_ctx,
+                           struct sysdb_ctx *sysdb,
+                           struct sss_domain_info *domain,
+                           const char *sub_filter,
+                           const char **attrs,
+                           size_t *msgs_count,
+                           struct ldb_message ***msgs)
+{
+    TALLOC_CTX *tmpctx;
+    struct ldb_dn *basedn;
+    char *filter;
+    int ret;
+
+    tmpctx = talloc_new(mem_ctx);
+    if (!tmpctx) {
+        return ENOMEM;
+    }
+
+    if (domain == NULL) {
+        domain = sysdb->domain;
+    }
+
+    basedn = ldb_dn_new_fmt(tmpctx, sysdb->ldb,
+                            SYSDB_TMPL_NETGROUP_BASE, domain->name);
+    if (!basedn) {
+        DEBUG(2, ("Failed to build base dn\n"));
+        ret = ENOMEM;
+        goto fail;
+    }
+
+    filter = talloc_asprintf(tmpctx, "(&(%s)%s)", SYSDB_NC, sub_filter);
+    if (!filter) {
+        DEBUG(2, ("Failed to build filter\n"));
+        ret = ENOMEM;
+        goto fail;
+    }
+
+    DEBUG(6, ("Search netgroups with filter: %s\n", filter));
+
+    ret = sysdb_search_entry(mem_ctx, sysdb, basedn,
+                             LDB_SCOPE_SUBTREE, filter, attrs,
+                             msgs_count, msgs);
+    if (ret) {
+        goto fail;
+    }
+
+    talloc_zfree(tmpctx);
+    return EOK;
+
+fail:
+    DEBUG(6, ("Error: %d (%s)\n", ret, strerror(ret)));
+    talloc_zfree(tmpctx);
+    return ret;
+}
+
 /* =Delete-Netgroup-by-Name============================================== */
 
 int sysdb_delete_netgroup(struct sysdb_ctx *sysdb,
