@@ -19,14 +19,13 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#define _GNU_SOURCE
+#include "util/util.h"
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/time.h>
 #include <sys/param.h>
 #include <time.h>
 #include <string.h>
-#include "config.h"
 #ifdef HAVE_SYS_INOTIFY_H
 #include <sys/inotify.h>
 #endif
@@ -40,7 +39,6 @@
 #include <arpa/nameser.h>
 #include <resolv.h>
 
-#include "util/util.h"
 #include "popt.h"
 #include "tevent.h"
 #include "confdb/confdb.h"
@@ -62,6 +60,8 @@
  */
 #define KRB5_RCACHE_DIR_DISABLE "__LIBKRB5_DEFAULTS__"
 
+int cmdline_debug_level;
+
 struct svc_spy;
 
 struct mt_svc {
@@ -124,6 +124,7 @@ struct mt_ctx {
     bool check_children;
     bool services_started;
     struct netlink_ctx *nlctx;
+    const char *conf_path;
 };
 
 static int start_service(struct mt_svc *mt_svc);
@@ -938,13 +939,30 @@ static int get_service_config(struct mt_ctx *ctx, const char *name,
     }
 
     if (!svc->command) {
-        svc->command = talloc_asprintf(svc, "%s/sssd_%s -d %d%s%s",
-                                       SSSD_LIBEXEC_PATH,
-                                       svc->name, debug_level,
-                                       (debug_timestamps?
-                                              "": " --debug-timestamps=0"),
-                                       (debug_to_file ?
-                                              " --debug-to-files":""));
+        if (cmdline_debug_level == SSS_UNRESOLVED_DEBUG_LEVEL) {
+            svc->command = talloc_asprintf(svc, "%s/sssd_%s %s%s",
+                                           SSSD_LIBEXEC_PATH,
+                                           svc->name,
+                                           debug_timestamps?
+                                                  "": "--debug-timestamps=0 ",
+                                           debug_to_file?
+                                                  "--debug-to-files":"");
+        } else {
+            /* If the debug level was specified at the command-line,
+             * make sure to pass it into the children, overriding the
+             * config file.
+             */
+            svc->command = talloc_asprintf(svc, "%s/sssd_%s -d %d%s%s",
+                                           SSSD_LIBEXEC_PATH,
+                                           svc->name,
+                                           cmdline_debug_level,
+                                           debug_timestamps ?
+                                               "" :
+                                               " --debug-timestamps=0",
+                                           debug_to_file ?
+                                               " --debug-to-files" :
+                                               "");
+        }
         if (!svc->command) {
             talloc_free(svc);
             return ENOMEM;
@@ -1067,12 +1085,25 @@ static int get_provider_config(struct mt_ctx *ctx, const char *name,
 
     /* if there are no custom commands, build a default one */
     if (!svc->command) {
-        svc->command = talloc_asprintf(svc,
-                            "%s/sssd_be -d %d%s%s --domain %s",
-                            SSSD_LIBEXEC_PATH, debug_level,
-                            (debug_timestamps?"": " --debug-timestamps=0"),
-                            (debug_to_file?" --debug-to-files":""),
-                            svc->name);
+        if (cmdline_debug_level == SSS_UNRESOLVED_DEBUG_LEVEL) {
+            svc->command = talloc_asprintf(svc,
+                                "%s/sssd_be --domain %s%s%s",
+                                SSSD_LIBEXEC_PATH,
+                                svc->name,
+                                debug_timestamps ? ""
+                                                 : " --debug-timestamps=0",
+                                debug_to_file ? " --debug-to-files" : "");
+        } else {
+            svc->command = talloc_asprintf(svc,
+                                "%s/sssd_be --domain %s -d %d%s%s ",
+                                SSSD_LIBEXEC_PATH,
+                                svc->name,
+                                cmdline_debug_level,
+                                debug_timestamps ? ""
+                                                 : " --debug-timestamps=0",
+                                debug_to_file ? " --debug-to-files" : "");
+        }
+
         if (!svc->command) {
             talloc_free(svc);
             return ENOMEM;
@@ -2337,6 +2368,11 @@ int main(int argc, const char *argv[])
         }
     }
 
+    /* If the level was passed at the command-line, we want
+     * to save it and pass it to the children later.
+     */
+    cmdline_debug_level = debug_level;
+
     if (opt_daemon && opt_interactive) {
         fprintf(stderr, "Option -i|--interactive is not allowed together with -D|--daemon\n");
         poptPrintUsage(pc, stderr, 0);
@@ -2411,7 +2447,8 @@ int main(int argc, const char *argv[])
     }
 
     /* set up things like debug , signals, daemonization, etc... */
-    ret = server_setup("sssd", flags, CONFDB_MONITOR_CONF_ENTRY, &main_ctx);
+    monitor->conf_path = CONFDB_MONITOR_CONF_ENTRY;
+    ret = server_setup("sssd", flags, monitor->conf_path, &main_ctx);
     if (ret != EOK) return 2;
 
     monitor->ev = main_ctx->event_ctx;
