@@ -27,6 +27,7 @@
 #include "util/util.h"
 #include "confdb/confdb.h"
 #include "sbus/sssd_dbus.h"
+#include "sbus/sbus_client.h"
 #include "monitor/monitor_interfaces.h"
 
 int monitor_get_sbus_address(TALLOC_CTX *mem_ctx, char **address)
@@ -177,10 +178,11 @@ int monitor_common_res_init(DBusMessage *message,
     return monitor_common_pong(message, conn);
 }
 
-int monitor_common_rotate_logs(DBusMessage *message,
-                               struct sbus_connection *conn)
+errno_t monitor_common_rotate_logs(struct confdb_ctx *confdb,
+                                   const char *conf_path)
 {
-    int ret;
+    errno_t ret;
+    int old_debug_level = debug_level;
 
     ret = rotate_debug_files();
     if (ret) {
@@ -189,5 +191,62 @@ int monitor_common_rotate_logs(DBusMessage *message,
         return ret;
     }
 
-    return monitor_common_pong(message, conn);
+    /* Get new debug level from the confdb */
+    ret = confdb_get_int(confdb, NULL, conf_path,
+                         CONFDB_SERVICE_DEBUG_LEVEL,
+                         old_debug_level,
+                         &debug_level);
+    if (ret != EOK) {
+        DEBUG(0, ("Error reading from confdb (%d) [%s]\n",
+                  ret, strerror(ret)));
+        /* Try to proceed with the old value */
+        debug_level = old_debug_level;
+    }
+
+    if (debug_level != old_debug_level) {
+        DEBUG(0, ("Debug level changed to %d\n", debug_level));
+    }
+
+    return EOK;
+}
+
+errno_t sss_monitor_init(TALLOC_CTX *mem_ctx,
+                         struct tevent_context *ev,
+                         struct sbus_interface *intf,
+                         const char *svc_name,
+                         uint16_t svc_version,
+                         void *pvt,
+                         struct sbus_connection **mon_conn)
+{
+    errno_t ret;
+    char *sbus_address;
+    struct sbus_connection *conn;
+
+    /* Set up SBUS connection to the monitor */
+    ret = monitor_get_sbus_address(NULL, &sbus_address);
+    if (ret != EOK) {
+        DEBUG(0, ("Could not locate monitor address.\n"));
+        return ret;
+    }
+
+    ret = sbus_client_init(mem_ctx, ev, sbus_address,
+                           intf, &conn,
+                           NULL, pvt);
+    if (ret != EOK) {
+        DEBUG(0, ("Failed to connect to monitor services.\n"));
+        talloc_free(sbus_address);
+        return ret;
+    }
+    talloc_free(sbus_address);
+
+    /* Identify ourselves to the monitor */
+    ret = monitor_common_send_id(conn, svc_name, svc_version);
+    if (ret != EOK) {
+        DEBUG(0, ("Failed to identify to the monitor!\n"));
+        return ret;
+    }
+
+    *mon_conn = conn;
+
+    return EOK;
 }
