@@ -57,13 +57,15 @@ static int data_provider_go_offline(DBusMessage *message,
                                     struct sbus_connection *conn);
 static int data_provider_reset_offline(DBusMessage *message,
                                        struct sbus_connection *conn);
+static int data_provider_logrotate(DBusMessage *message,
+                                struct sbus_connection *conn);
 
 struct sbus_method monitor_be_methods[] = {
     { MON_CLI_METHOD_PING, monitor_common_pong },
     { MON_CLI_METHOD_RES_INIT, data_provider_res_init },
     { MON_CLI_METHOD_OFFLINE, data_provider_go_offline },
     { MON_CLI_METHOD_RESET_OFFLINE, data_provider_reset_offline },
-    { MON_CLI_METHOD_ROTATE, monitor_common_rotate_logs },
+    { MON_CLI_METHOD_ROTATE, data_provider_logrotate },
     { NULL, NULL }
 };
 
@@ -76,13 +78,11 @@ struct sbus_interface monitor_be_interface = {
 };
 
 static int client_registration(DBusMessage *message, struct sbus_connection *conn);
-static int be_check_online(DBusMessage *message, struct sbus_connection *conn);
 static int be_get_account_info(DBusMessage *message, struct sbus_connection *conn);
 static int be_pam_handler(DBusMessage *message, struct sbus_connection *conn);
 
 struct sbus_method be_methods[] = {
     { DP_METHOD_REGISTER, client_registration },
-    { DP_METHOD_ONLINE, be_check_online },
     { DP_METHOD_GETACCTINFO, be_get_account_info },
     { DP_METHOD_PAMHANDLER, be_pam_handler },
     { NULL, NULL }
@@ -180,53 +180,6 @@ static void be_reset_offline(struct be_ctx *ctx)
     be_run_online_cb(ctx);
 }
 
-static int be_check_online(DBusMessage *message, struct sbus_connection *conn)
-{
-    struct be_client *becli;
-    DBusMessage *reply;
-    DBusConnection *dbus_conn;
-    dbus_bool_t dbret;
-    void *user_data;
-    dbus_uint16_t online;
-    dbus_uint16_t err_maj = 0;
-    dbus_uint32_t err_min = 0;
-    static const char *err_msg = "Success";
-
-    user_data = sbus_conn_get_private_data(conn);
-    if (!user_data) return EINVAL;
-    becli = talloc_get_type(user_data, struct be_client);
-    if (!becli) return EINVAL;
-
-    reply = dbus_message_new_method_return(message);
-    if (!reply) return ENOMEM;
-
-    if (be_is_offline(becli->bectx)) {
-        online = MOD_OFFLINE;
-    } else {
-        online = MOD_ONLINE;
-    }
-
-    dbret = dbus_message_append_args(reply,
-                                     DBUS_TYPE_UINT16, &online,
-                                     DBUS_TYPE_UINT16, &err_maj,
-                                     DBUS_TYPE_UINT32, &err_min,
-                                     DBUS_TYPE_STRING, &err_msg,
-                                     DBUS_TYPE_INVALID);
-    if (!dbret) {
-        DEBUG(1, ("Failed to generate dbus reply\n"));
-        return EIO;
-    }
-
-    dbus_conn = sbus_get_connection(becli->conn);
-    dbus_connection_send(dbus_conn, reply, NULL);
-    dbus_message_unref(reply);
-
-    DEBUG(4, ("Request processed. Returned %d,%d,%s\n",
-              err_maj, err_min, err_msg));
-
-    return EOK;
-}
-
 static char *dp_pam_err_to_string(TALLOC_CTX *memctx, int dp_err_type, int errnum)
 {
     switch (dp_err_type) {
@@ -917,40 +870,6 @@ static int be_srv_init(struct be_ctx *ctx)
     return EOK;
 }
 
-/* mon_cli_init
- * sbus channel to the monitor daemon */
-static int mon_cli_init(struct be_ctx *ctx)
-{
-    char *sbus_address;
-    int ret;
-
-    /* Set up SBUS connection to the monitor */
-    ret = monitor_get_sbus_address(ctx, &sbus_address);
-    if (ret != EOK) {
-        DEBUG(0, ("Could not locate monitor address.\n"));
-        return ret;
-    }
-
-    ret = sbus_client_init(ctx, ctx->ev, sbus_address,
-                           &monitor_be_interface, &ctx->mon_conn,
-                           NULL, ctx);
-    if (ret != EOK) {
-        DEBUG(0, ("Failed to connect to monitor services.\n"));
-        return ret;
-    }
-
-    /* Identify ourselves to the monitor */
-    ret = monitor_common_send_id(ctx->mon_conn,
-                                 ctx->identity,
-                                 DATA_PROVIDER_VERSION);
-    if (ret != EOK) {
-        DEBUG(0, ("Failed to identify to the monitor!\n"));
-        return ret;
-    }
-
-    return EOK;
-}
-
 static void be_target_access_permit(struct be_req *be_req)
 {
     struct pam_data *pd = talloc_get_type(be_req->req_data, struct pam_data);
@@ -1187,7 +1106,9 @@ int be_process_init(TALLOC_CTX *mem_ctx,
         return ret;
     }
 
-    ret = mon_cli_init(ctx);
+    ret = sss_monitor_init(ctx, ctx->ev, &monitor_be_interface,
+                           ctx->identity, DATA_PROVIDER_VERSION,
+                           ctx, &ctx->mon_conn);
     if (ret != EOK) {
         DEBUG(0, ("fatal error setting up monitor bus\n"));
         return ret;
@@ -1264,8 +1185,8 @@ int main(int argc, const char *argv[])
     poptContext pc;
     char *be_domain = NULL;
     char *srv_name = NULL;
-    char *conf_entry = NULL;
     struct main_context *main_ctx;
+    char *confdb_path;
     int ret;
 
     struct poptOption long_options[] = {
@@ -1303,10 +1224,10 @@ int main(int argc, const char *argv[])
     srv_name = talloc_asprintf(NULL, "sssd[be[%s]]", be_domain);
     if (!srv_name) return 2;
 
-    conf_entry = talloc_asprintf(NULL, CONFDB_DOMAIN_PATH_TMPL, be_domain);
-    if (!conf_entry) return 2;
+    confdb_path = talloc_asprintf(NULL, CONFDB_DOMAIN_PATH_TMPL, be_domain);
+    if (!confdb_path) return 2;
 
-    ret = server_setup(srv_name, 0, conf_entry, &main_ctx);
+    ret = server_setup(srv_name, 0, confdb_path, &main_ctx);
     if (ret != EOK) {
         DEBUG(0, ("Could not set up mainloop [%d]\n", ret));
         return 2;
@@ -1364,3 +1285,16 @@ static int data_provider_reset_offline(DBusMessage *message,
     check_if_online(be_ctx);
     return monitor_common_pong(message, conn);
 }
+
+static int data_provider_logrotate(DBusMessage *message,
+                                struct sbus_connection *conn)
+{
+    errno_t ret;
+    struct be_ctx *be_ctx = talloc_get_type(sbus_conn_get_private_data(conn),
+                                            struct be_ctx);
+
+    ret = monitor_common_rotate_logs(be_ctx->cdb, be_ctx->conf_path);
+    if (ret != EOK) return ret;
+
+    return monitor_common_pong(message, conn);
+}
