@@ -65,38 +65,18 @@ int be_fo_is_srv_identifier(const char *server)
 static int be_fo_get_options(TALLOC_CTX *mem_ctx, struct be_ctx *ctx,
                              struct fo_options *opts)
 {
-    char *str_opt;
-    int ret;
+    errno_t ret;
 
     /* todo get timeout from configuration */
     opts->retry_timeout = 30;
     opts->srv_retry_timeout = 14400;
 
-    ret = confdb_get_string(ctx->cdb, mem_ctx, ctx->conf_path,
-                            CONFDB_DOMAIN_FAMILY_ORDER,
-                            "ipv4_first", &str_opt);
+    ret = resolv_get_family_order(ctx->cdb, ctx->conf_path,
+                                  &opts->family_order);
     if (ret != EOK) {
         return ret;
     }
 
-    DEBUG(7, ("Lookup order: %s\n", str_opt));
-
-    if (strcasecmp(str_opt, "ipv4_first") == 0) {
-        opts->family_order = IPV4_FIRST;
-    } else if (strcasecmp(str_opt, "ipv4_only") == 0) {
-        opts->family_order = IPV4_ONLY;
-    } else if (strcasecmp(str_opt, "ipv6_first") == 0) {
-        opts->family_order = IPV6_FIRST;
-    } else if (strcasecmp(str_opt, "ipv6_only") == 0) {
-        opts->family_order = IPV6_ONLY;
-    } else {
-        DEBUG(1, ("Unknown value for option %s: %s\n",
-                  CONFDB_DOMAIN_FAMILY_ORDER, str_opt));
-        talloc_free(str_opt);
-        return EINVAL;
-    }
-
-    talloc_free(str_opt);
     return EOK;
 }
 
@@ -117,7 +97,7 @@ int be_init_failover(struct be_ctx *ctx)
 
     ret = confdb_get_int(ctx->cdb, ctx, ctx->conf_path,
                          CONFDB_DOMAIN_RESOLV_TIMEOUT,
-                         5, &fo_timeout);
+                         RESOLV_DEFAULT_TIMEOUT, &fo_timeout);
     if (ret != EOK) {
         return ret;
     }
@@ -426,7 +406,7 @@ static void be_resolve_server_done(struct tevent_req *subreq)
             return;
         }
         DEBUG(6, ("Couldn't resolve server (%s), resolver returned (%d)\n",
-                  fo_get_server_name(state->srv), ret));
+                  fo_get_server_str_name(state->srv), ret));
 
         state->attempts++;
         if (state->attempts >= 10) {
@@ -452,13 +432,13 @@ static void be_resolve_server_done(struct tevent_req *subreq)
 
     /* all fine we got the server */
 
-    if (debug_level >= 4) {
+    if (debug_level >= 4 && fo_get_server_name(state->srv)) {
         struct resolv_hostent *srvaddr;
         char ipaddr[128];
         srvaddr = fo_get_server_hostent(state->srv);
         if (!srvaddr) {
             DEBUG(3, ("FATAL: No hostent available for server (%s)\n",
-                      fo_get_server_name(state->srv)));
+                      fo_get_server_str_name(state->srv)));
             tevent_req_error(req, EFAULT);
             return;
         }
@@ -467,7 +447,7 @@ static void be_resolve_server_done(struct tevent_req *subreq)
                   ipaddr, 128);
 
         DEBUG(4, ("Found address for server %s: [%s] TTL %d\n",
-                  fo_get_server_name(state->srv), ipaddr,
+                  fo_get_server_str_name(state->srv), ipaddr,
                   srvaddr->addr_list[0]->ttl));
     }
 
