@@ -917,23 +917,16 @@ static void krb5_child_done(struct tevent_req *subreq)
     if (kr->old_ccname != NULL) {
         ret = safe_remove_old_ccache_file(kr->old_ccname, kr->ccname);
         if (ret != EOK) {
-            DEBUG(1, ("Failed to remove old ccache file [%s], please remove it manually.\n"));
+            DEBUG(1, ("Failed to remove old ccache file [%s], "
+                      "please remove it manually.\n", kr->old_ccname));
         }
     }
 
-    struct sysdb_attrs *attrs;
-    attrs = sysdb_new_attrs(state);
-    ret = sysdb_attrs_add_string(attrs, SYSDB_CCACHE_FILE, kr->ccname);
-    if (ret != EOK) {
-        DEBUG(1, ("sysdb_attrs_add_string failed.\n"));
-        goto done;
-    }
-
     ret = krb5_save_ccname(state, state->be_ctx->sysdb,
                            state->be_ctx->domain,
                            pd->user, kr->ccname);
     if (ret) {
-        DEBUG(1, ("krb5_save_ccname_send failed.\n"));
+        DEBUG(1, ("krb5_save_ccname failed.\n"));
         goto done;
     }
 
