@@ -30,23 +30,17 @@
 
 #include "util/sss_krb5.h"
 #include "providers/dp_backend.h"
+#include "providers/child_common.h"
 #include "providers/krb5/krb5_common.h"
 
 #define CCACHE_ENV_NAME "KRB5CCNAME"
-#define SSSD_KRB5_CHANGEPW_PRINCIPAL "SSSD_KRB5_CHANGEPW_PRINCIPAL"
 
 #define ILLEGAL_PATH_PATTERN "//|/\\./|/\\.\\./"
 
 struct krb5child_req {
-    pid_t child_pid;
-    int read_from_child_fd;
-    int write_to_child_fd;
-
     struct pam_data *pd;
     struct krb5_ctx *krb5_ctx;
 
-    struct tevent_timer *timeout_handler;
-
     const char *ccname;
     const char *old_ccname;
     const char *homedir;
@@ -58,8 +52,12 @@ struct krb5child_req {
     struct fo_server *kpasswd_srv;
     bool active_ccache_present;
     bool valid_tgt_present;
+    bool run_as_user;
 };
 
+errno_t krb5_setup(TALLOC_CTX *mem_ctx, struct pam_data *pd,
+                   struct krb5_ctx *krb5_ctx, struct krb5child_req **krb5_req);
+
 void krb5_pam_handler(struct be_req *be_req);
 
 struct tevent_req *krb5_auth_send(TALLOC_CTX *mem_ctx,
@@ -68,6 +66,13 @@ struct tevent_req *krb5_auth_send(TALLOC_CTX *mem_ctx,
                                   struct pam_data *pd,
                                   struct krb5_ctx *krb5_ctx);
 int krb5_auth_recv(struct tevent_req *req, int *pam_status, int *dp_err);
+void krb5_auth_done(struct tevent_req *req);
+
+struct tevent_req *handle_child_send(TALLOC_CTX *mem_ctx,
+                                     struct tevent_context *ev,
+                                     struct krb5child_req *kr);
+int handle_child_recv(struct tevent_req *req, TALLOC_CTX *mem_ctx,
+                      uint8_t **buf, ssize_t *len);
 
 errno_t add_user_to_delayed_online_authentication(struct krb5_ctx *krb5_ctx,
                                                   struct pam_data *pd,
@@ -75,4 +80,23 @@ errno_t add_user_to_delayed_online_authentication(struct krb5_ctx *krb5_ctx,
 errno_t init_delayed_online_authentication(struct krb5_ctx *krb5_ctx,
                                            struct be_ctx *be_ctx,
                                            struct tevent_context *ev);
+
+errno_t init_renew_tgt(struct krb5_ctx *krb5_ctx, struct be_ctx *be_ctx,
+                       struct tevent_context *ev, time_t renew_intv);
+errno_t add_tgt_to_renew_table(struct krb5_ctx *krb5_ctx, const char *ccfile,
+                               struct tgt_times *tgtt, struct pam_data *pd,
+                               const char *upn);
+
+/* krb5_access.c */
+struct tevent_req *krb5_access_send(TALLOC_CTX *mem_ctx,
+                                    struct tevent_context *ev,
+                                    struct be_ctx *be_ctx,
+                                    struct pam_data *pd,
+                                    struct krb5_ctx *krb5_ctx);
+int krb5_access_recv(struct tevent_req *req, bool *access_allowed);
+
+/* krb5_wait_queue.c */
+errno_t add_to_wait_queue(struct be_req *be_req, struct pam_data *pd,
+                          struct krb5_ctx *krb5_ctx);
+void check_wait_queue(struct krb5_ctx *krb5_ctx, char *username);
 #endif /* __KRB5_AUTH_H__ */
