@@ -44,7 +44,8 @@ struct dp_option default_krb5_opts[] = {
     { "krb5_renewable_lifetime", DP_OPT_STRING, NULL_STRING, NULL_STRING },
     { "krb5_lifetime", DP_OPT_STRING, NULL_STRING, NULL_STRING },
     { "krb5_renew_interval", DP_OPT_NUMBER, NULL_NUMBER, NULL_NUMBER },
-    { "krb5_use_fast", DP_OPT_STRING, NULL_STRING, NULL_STRING }
+    { "krb5_use_fast", DP_OPT_STRING, NULL_STRING, NULL_STRING },
+    { "krb5_fast_principal", DP_OPT_STRING, NULL_STRING, NULL_STRING }
 };
 
 errno_t check_and_export_lifetime(struct dp_option *opts, const int opt_id,
@@ -109,6 +110,7 @@ errno_t check_and_export_options(struct dp_option *opts,
     const char *realm;
     const char *dummy;
     char *use_fast_str;
+    char *fast_principal;
 
     realm = dp_opt_get_cstring(opts, KRB5_REALM);
     if (realm == NULL) {
@@ -155,6 +157,14 @@ errno_t check_and_export_options(struct dp_option *opts,
             ret = setenv(SSSD_KRB5_USE_FAST, use_fast_str, 1);
             if (ret != EOK) {
                 DEBUG(2, ("setenv [%s] failed.\n", SSSD_KRB5_USE_FAST));
+            } else {
+                fast_principal = dp_opt_get_string(opts, KRB5_FAST_PRINCIPAL);
+                if (fast_principal != NULL) {
+                    ret = setenv(SSSD_KRB5_FAST_PRINCIPAL, fast_principal, 1);
+                    if (ret != EOK) {
+                        DEBUG(2, ("setenv [%s] failed.\n", SSSD_KRB5_FAST_PRINCIPAL));
+                    }
+                }
             }
         }
     }
@@ -369,43 +379,62 @@ static void krb5_resolve_callback(void *private_data, struct fo_server *server)
     struct krb5_service *krb5_service;
     struct resolv_hostent *srvaddr;
     char *address;
+    char *safe_address;
     int ret;
+    TALLOC_CTX *tmp_ctx = NULL;
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        DEBUG(1, ("talloc_new failed\n"));
+        return;
+    }
 
     krb5_service = talloc_get_type(private_data, struct krb5_service);
     if (!krb5_service) {
         DEBUG(1, ("FATAL: Bad private_data\n"));
+        talloc_free(tmp_ctx);
         return;
     }
 
     srvaddr = fo_get_server_hostent(server);
     if (!srvaddr) {
         DEBUG(1, ("FATAL: No hostent available for server (%s)\n",
-                  fo_get_server_name(server)));
+                  fo_get_server_str_name(server)));
+        talloc_free(tmp_ctx);
         return;
     }
 
-    address = resolv_get_string_address(krb5_service, srvaddr);
+    address = resolv_get_string_address(tmp_ctx, srvaddr);
     if (address == NULL) {
         DEBUG(1, ("resolv_get_string_address failed.\n"));
+        talloc_free(tmp_ctx);
         return;
     }
 
-    address = talloc_asprintf_append(address, ":%d",
-                                     fo_get_server_port(server));
-    if (address == NULL) {
+    safe_address = sss_escape_ip_address(tmp_ctx,
+                                         srvaddr->family,
+                                         address);
+    if (safe_address == NULL) {
+        DEBUG(1, ("sss_escape_ip_address failed.\n"));
+        talloc_free(tmp_ctx);
+        return;
+    }
+
+    safe_address = talloc_asprintf_append(safe_address, ":%d",
+                                          fo_get_server_port(server));
+    if (safe_address == NULL) {
         DEBUG(1, ("talloc_asprintf_append failed.\n"));
+        talloc_free(tmp_ctx);
         return;
     }
 
-    talloc_zfree(krb5_service->address);
-    krb5_service->address = address;
-
-    ret = write_krb5info_file(krb5_service->realm, address,
+    ret = write_krb5info_file(krb5_service->realm, safe_address,
                               krb5_service->name);
     if (ret != EOK) {
         DEBUG(2, ("write_krb5info_file failed, authentication might fail.\n"));
     }
 
+    talloc_free(tmp_ctx);
     return;
 }
 
