@@ -38,6 +38,7 @@
 #define SSSD_KRB5_RENEWABLE_LIFETIME "SSSD_KRB5_RENEWABLE_LIFETIME"
 #define SSSD_KRB5_LIFETIME "SSSD_KRB5_LIFETIME"
 #define SSSD_KRB5_USE_FAST "SSSD_KRB5_USE_FAST"
+#define SSSD_KRB5_FAST_PRINCIPAL "SSSD_KRB5_FAST_PRINCIPAL"
 
 #define KDCINFO_TMPL PUBCONF_PATH"/kdcinfo.%s"
 #define KPASSWDINFO_TMPL PUBCONF_PATH"/kpasswdinfo.%s"
@@ -59,6 +60,7 @@ enum krb5_opts {
     KRB5_LIFETIME,
     KRB5_RENEW_INTERVAL,
     KRB5_USE_FAST,
+    KRB5_FAST_PRINCIPAL,
 
     KRB5_OPTS
 };
@@ -74,7 +76,6 @@ struct tgt_times {
 
 struct krb5_service {
     char *name;
-    char *address;
     char *realm;
 };
 
