@@ -904,7 +904,7 @@ static void sdap_pam_chpass_done(struct tevent_req *req)
 
     ret = sdap_exop_modify_passwd_recv(req, state, &result, &user_error_message);
     talloc_zfree(req);
-    if (ret) {
+    if (ret && ret != EIO) {
         state->pd->pam_status = PAM_SYSTEM_ERR;
         goto done;
     }
@@ -914,19 +914,24 @@ static void sdap_pam_chpass_done(struct tevent_req *req)
         state->pd->pam_status = PAM_SUCCESS;
         dp_err = DP_ERR_OK;
         break;
+    case SDAP_AUTH_PW_CONSTRAINT_VIOLATION:
+        state->pd->pam_status = PAM_NEW_AUTHTOK_REQD;
+        break;
     default:
         state->pd->pam_status = PAM_AUTHTOK_ERR;
-        if (user_error_message != NULL) {
-            ret = pack_user_info_chpass_error(state->pd, user_error_message,
-                                              &msg_len, &msg);
+        break;
+    }
+
+    if (state->pd->pam_status != PAM_SUCCESS && user_error_message != NULL) {
+        ret = pack_user_info_chpass_error(state->pd, user_error_message,
+                                            &msg_len, &msg);
+        if (ret != EOK) {
+            DEBUG(1, ("pack_user_info_chpass_error failed.\n"));
+        } else {
+            ret = pam_add_response(state->pd, SSS_PAM_USER_INFO, msg_len,
+                                    msg);
             if (ret != EOK) {
-                DEBUG(1, ("pack_user_info_chpass_error failed.\n"));
-            } else {
-                ret = pam_add_response(state->pd, SSS_PAM_USER_INFO, msg_len,
-                                       msg);
-                if (ret != EOK) {
-                    DEBUG(1, ("pam_add_response failed.\n"));
-                }
+                DEBUG(1, ("pam_add_response failed.\n"));
             }
         }
     }
