@@ -65,6 +65,7 @@ struct tevent_req *users_get_send(TALLOC_CTX *memctx,
     struct users_get_state *state;
     const char *attr_name;
     char *clean_name;
+    char *base_filter;
     int ret;
 
     req = tevent_req_create(memctx, &state, struct users_get_state);
@@ -103,15 +104,24 @@ struct tevent_req *users_get_send(TALLOC_CTX *memctx,
         goto fail;
     }
 
-    state->filter = talloc_asprintf(state, "(&(%s=%s)(objectclass=%s))",
-                                    attr_name, clean_name,
-                                    ctx->opts->user_map[SDAP_OC_USER].name);
+    base_filter = talloc_asprintf(state, "(&(%s=%s)(objectclass=%s))",
+                                  attr_name, clean_name,
+                                  ctx->opts->user_map[SDAP_OC_USER].name);
+    talloc_zfree(clean_name);
+    if (!base_filter) {
+        DEBUG(2, ("Failed to build the base filter\n"));
+        ret = ENOMEM;
+        goto fail;
+    }
+
+    state->filter = sdap_get_id_specific_filter(state, base_filter,
+           dp_opt_get_string(ctx->opts->basic, SDAP_USER_SEARCH_FILTER));
+    talloc_zfree(base_filter);
     if (!state->filter) {
-        DEBUG(2, ("Failed to build filter\n"));
+        DEBUG(2, ("Failed to build user filter\n"));
         ret = ENOMEM;
         goto fail;
     }
-    talloc_zfree(clean_name);
 
     /* TODO: handle attrs_type */
     ret = build_attrs_from_map(state, ctx->opts->user_map,
@@ -297,6 +307,7 @@ struct tevent_req *groups_get_send(TALLOC_CTX *memctx,
     struct groups_get_state *state;
     const char *attr_name;
     char *clean_name;
+    char *base_filter;
     int ret;
 
     req = tevent_req_create(memctx, &state, struct groups_get_state);
@@ -335,20 +346,28 @@ struct tevent_req *groups_get_send(TALLOC_CTX *memctx,
         goto fail;
     }
 
-    state->filter =
-            talloc_asprintf(state,
-                            "(&(%s=%s)(objectclass=%s)(%s=*)(&(%s=*)(!(%s=0))))",
-                            attr_name, clean_name,
-                            ctx->opts->group_map[SDAP_OC_GROUP].name,
-                            ctx->opts->group_map[SDAP_AT_GROUP_NAME].name,
-                            ctx->opts->group_map[SDAP_AT_GROUP_GID].name,
-                            ctx->opts->group_map[SDAP_AT_GROUP_GID].name);
-    if (!state->filter) {
+    base_filter = talloc_asprintf(state,
+                                "(&(%s=%s)(objectclass=%s)(%s=*)(&(%s=*)(!(%s=0))))",
+                                attr_name, clean_name,
+                                ctx->opts->group_map[SDAP_OC_GROUP].name,
+                                ctx->opts->group_map[SDAP_AT_GROUP_NAME].name,
+                                ctx->opts->group_map[SDAP_AT_GROUP_GID].name,
+                                ctx->opts->group_map[SDAP_AT_GROUP_GID].name);
+    talloc_zfree(clean_name);
+    if (!base_filter) {
         DEBUG(2, ("Failed to build filter\n"));
         ret = ENOMEM;
         goto fail;
     }
-    talloc_zfree(clean_name);
+
+    state->filter = sdap_get_id_specific_filter(state, base_filter,
+           dp_opt_get_string(ctx->opts->basic, SDAP_GROUP_SEARCH_FILTER));
+    talloc_zfree(base_filter);
+    if (!state->filter) {
+        DEBUG(2, ("Failed to build group-specific filter\n"));
+        ret = ENOMEM;
+        goto fail;
+    }
 
     /* TODO: handle attrs_type */
     ret = build_attrs_from_map(state, ctx->opts->group_map,
@@ -693,8 +712,13 @@ static void sdap_check_online_done(struct tevent_req *req)
     int ret;
     int dp_err = DP_ERR_FATAL;
     bool can_retry;
+    struct sdap_id_ctx *ctx;
+    struct sdap_server_opts *srv_opts;
 
-    ret = sdap_cli_connect_recv(req, NULL, &can_retry, NULL, NULL);
+    ctx = talloc_get_type(be_req->be_ctx->bet_info[BET_ID].pvt_bet_data,
+                          struct sdap_id_ctx);
+
+    ret = sdap_cli_connect_recv(req, NULL, &can_retry, NULL, &srv_opts);
     talloc_zfree(req);
 
     if (ret != EOK) {
@@ -703,6 +727,19 @@ static void sdap_check_online_done(struct tevent_req *req)
         }
     } else {
         dp_err = DP_ERR_OK;
+
+        if (!ctx->srv_opts) {
+            srv_opts->max_user_value = 0;
+            srv_opts->max_group_value = 0;
+        } else if (strcmp(srv_opts->server_id, ctx->srv_opts->server_id) == 0
+                   && srv_opts->supports_usn
+                   && ctx->srv_opts->last_usn > srv_opts->last_usn) {
+            ctx->srv_opts->max_user_value = 0;
+            ctx->srv_opts->max_group_value = 0;
+            ctx->srv_opts->last_usn = srv_opts->last_usn;
+        }
+
+        sdap_steal_server_opts(ctx, &srv_opts);
     }
 
     sdap_handler_done(be_req, dp_err, 0, NULL);
