@@ -123,6 +123,9 @@ struct sdap_ppolicy_data {
 
 #define SYSDB_AD_ACCOUNT_EXPIRES "adAccountExpires"
 #define SYSDB_AD_USER_ACCOUNT_CONTROL "adUserAccountControl"
+#define SYSDB_NDS_LOGIN_DISABLED "ndsLoginDisabled"
+#define SYSDB_NDS_LOGIN_EXPIRATION_TIME "ndsLoginExpirationTime"
+#define SYSDB_NDS_LOGIN_ALLOWED_TIME_MAP "ndsLoginAllowedTimeMap"
 
 #define SDAP_ROOTDSE_ATTR_NAMING_CONTEXTS "namingContexts"
 #define SDAP_ROOTDSE_ATTR_DEFAULT_NAMING_CONTEXT "defaultNamingContext"
@@ -175,6 +178,7 @@ enum sdap_basic_opt {
     SDAP_ID_TLS,
     SDAP_SASL_MECH,
     SDAP_SASL_AUTHID,
+    SDAP_SASL_REALM,
     SDAP_KRB5_KEYTAB,
     SDAP_KRB5_KINIT,
     SDAP_KRB5_KDC,
@@ -195,6 +199,7 @@ enum sdap_basic_opt {
     SDAP_ENUM_SEARCH_TIMEOUT,
     SDAP_DISABLE_AUTH_TLS,
     SDAP_PAGE_SIZE,
+    SDAP_DEREF_THRESHOLD,
     SDAP_SASL_CANONICALIZE,
 
     SDAP_OPTS_BASIC /* opts counter */
@@ -238,6 +243,10 @@ enum sdap_user_attrs {
     SDAP_AT_AD_ACCOUNT_EXPIRES,
     SDAP_AT_AD_USER_ACCOUNT_CONTROL,
     SDAP_AT_NS_ACCOUNT_LOCK,
+    SDAP_AT_AUTHORIZED_HOST,
+    SDAP_AT_NDS_LOGIN_DISABLED,
+    SDAP_AT_NDS_LOGIN_EXPIRATION_TIME,
+    SDAP_AT_NDS_LOGIN_ALLOWED_TIME_MAP,
 
     SDAP_OPTS_USER /* attrs counter */
 };
@@ -299,12 +308,23 @@ struct sdap_options {
 struct sdap_server_opts {
     char *server_id;
     bool supports_usn;
+    unsigned long last_usn;
     char *max_user_value;
     char *max_group_value;
 };
 
 struct sdap_id_ctx;
 
+struct sdap_attr_map_info {
+    struct sdap_attr_map *map;
+    int num_attrs;
+};
+
+struct sdap_deref_attrs {
+    struct sdap_attr_map *map;
+    struct sysdb_attrs *attrs;
+};
+
 int sdap_get_map(TALLOC_CTX *memctx,
                  struct confdb_ctx *cdb,
                  const char *conf_path,
@@ -325,6 +345,13 @@ int sdap_parse_group(TALLOC_CTX *memctx, struct sdap_options *opts,
                      struct sdap_handle *sh, struct sdap_msg *sm,
                      struct sysdb_attrs **_attrs, char **_dn);
 
+errno_t sdap_parse_deref(TALLOC_CTX *mem_ctx,
+                         struct sdap_attr_map_info *minfo,
+                         size_t num_maps,
+                         struct sdap_handle *sh,
+                         LDAPDerefRes *dref,
+                         struct sdap_deref_attrs ***_res);
+
 int sdap_get_msg_dn(TALLOC_CTX *memctx, struct sdap_handle *sh,
                     struct sdap_msg *sm, char **_dn);
 
@@ -346,7 +373,6 @@ bool sdap_check_sup_list(struct sup_list *l, const char *val);
 int build_attrs_from_map(TALLOC_CTX *memctx,
                          struct sdap_attr_map *map,
                          size_t size, const char ***_attrs);
-int append_attrs_to_array(const char **attrs, size_t size, const char *attr);
 
 int sdap_control_create(struct sdap_handle *sh, const char *oid, int iscritical,
                         struct berval *value, int dupval, LDAPControl **ctrlp);
