@@ -22,6 +22,9 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#define _XOPEN_SOURCE 500 /* for strptime() */
+#include <time.h>
+#undef _XOPEN_SOURCE
 #include <sys/param.h>
 #include <security/pam_modules.h>
 #include <talloc.h>
@@ -85,6 +88,17 @@ static struct tevent_req *sdap_access_service_send(
 
 static void sdap_account_expired_done(struct tevent_req *subreq);
 
+static errno_t sdap_access_host_recv(struct tevent_req *req,
+                                        int *pam_status);
+static void sdap_access_host_done(struct tevent_req *subreq);
+
+static struct tevent_req *sdap_access_host_send(
+        TALLOC_CTX *mem_ctx,
+        struct tevent_context *ev,
+        struct sdap_access_ctx *access_ctx,
+        struct pam_data *pd,
+        struct ldb_message *user_entry);
+
 static void sdap_access_done(struct tevent_req *req);
 void sdap_pam_access_handler(struct be_req *breq)
 {
@@ -259,6 +273,18 @@ static errno_t select_next_rule(struct tevent_req *req)
             tevent_req_set_callback(subreq, sdap_access_service_done, req);
             return EOK;
 
+        case LDAP_ACCESS_HOST:
+            subreq = sdap_access_host_send(state, state->ev,
+                                              state->access_ctx,
+                                              state->pd,
+                                              state->user_entry);
+            if (subreq == NULL) {
+                DEBUG(1, ("sdap_access_host_send failed.\n"));
+                return ENOMEM;
+            }
+            tevent_req_set_callback(subreq, sdap_access_host_done, req);
+            return EOK;
+
         default:
             DEBUG(1, ("Unexpected access rule type. Access denied.\n"));
     }
@@ -447,6 +473,174 @@ static errno_t sdap_account_expired_rhds(struct pam_data *pd,
     return EOK;
 }
 
+#define NDS_DISABLE_MSG "The user account is disabled on the server"
+#define NDS_EXPIRED_MSG "The user account is expired"
+#define NDS_TIME_MAP_MSG "The user account is not allowed at this time"
+
+static bool nds_check_expired(const char *exp_time_str)
+{
+    char *end;
+    struct tm tm;
+    time_t expire_time;
+    time_t now;
+
+    if (exp_time_str == NULL) {
+        DEBUG(9, ("ndsLoginExpirationTime is not set, access granted.\n"));
+        return false;
+    }
+
+    memset(&tm, 0, sizeof(tm));
+
+    end = strptime(exp_time_str, "%Y%m%d%H%M%SZ", &tm);
+    if (end == NULL) {
+        DEBUG(1, ("NDS expire date [%s] invalid.\n", exp_time_str));
+        return true;
+    }
+    if (*end != '\0') {
+        DEBUG(1, ("NDS expire date [%s] contains extra characters.\n",
+                  exp_time_str));
+        return true;
+    }
+
+    expire_time = mktime(&tm);
+    if (expire_time == -1) {
+        DEBUG(1, ("mktime failed to convert [%s].\n", exp_time_str));
+        return true;
+    }
+
+    tzset();
+    expire_time -= timezone;
+    now = time(NULL);
+    DEBUG(9, ("Time info: tzname[0] [%s] tzname[1] [%s] timezone [%d] "
+              "daylight [%d] now [%d] expire_time [%d].\n", tzname[0],
+              tzname[1], timezone, daylight, now, expire_time));
+
+    if (difftime(now, expire_time) > 0.0) {
+        DEBUG(4, ("NDS account expired.\n"));
+        return true;
+    }
+
+    return false;
+}
+
+/* There is no real documentation of the byte string value of
+ * loginAllowedTimeMap, but some good example code in
+ * http://http://developer.novell.com/documentation/samplecode/extjndi_sample/CheckBind.java.html
+ */
+static bool nds_check_time_map(const struct ldb_val *time_map)
+{
+    time_t now;
+    struct tm *tm_now;
+    size_t map_index;
+    div_t q;
+    uint8_t mask = 0;
+
+    if (time_map == NULL) {
+        DEBUG(9, ("loginAllowedTimeMap is missing, access granted.\n"));
+        return false;
+    }
+
+    if (time_map->length != 42) {
+        DEBUG(4, ("Allowed time map has the wrong size, "
+                  "got [%d], expected 42.\n", time_map->length));
+        return true;
+    }
+
+    now = time(NULL);
+    tm_now = gmtime(&now);
+
+    map_index = tm_now->tm_wday * 48 + tm_now->tm_hour * 2 +
+                (tm_now->tm_min < 30 ? 0 : 1);
+
+    if (map_index > 335) {
+        DEBUG(1, ("Unexpected index value [%d] for time map.\n", index));
+        return true;
+    }
+
+    q = div(map_index, 8);
+
+    if (q.quot > 41 || q.quot < 0 || q.rem > 7 || q.rem < 0) {
+        DEBUG(1, ("Unexpected result of div(), [%d][%d][%d].\n",
+                  index, q.quot, q.rem));
+        return true;
+    }
+
+    if (q.rem > 0) {
+        mask = 1 << q.rem;
+    }
+
+    if (time_map->data[q.quot] & mask) {
+        DEBUG(4, ("Access allowed by time map.\n"));
+        return false;
+    }
+
+    return true;
+}
+
+static errno_t sdap_account_expired_nds(struct pam_data *pd,
+                                         struct ldb_message *user_entry,
+                                         int *pam_status)
+{
+    bool locked = true;
+    int ret;
+    const char *exp_time_str;
+    const struct ldb_val *time_map;
+
+    DEBUG(6, ("Performing NDS access check for user [%s]\n", pd->user));
+
+    locked = ldb_msg_find_attr_as_bool(user_entry, SYSDB_NDS_LOGIN_DISABLED,
+                                       false);
+    DEBUG(9, ("Account for user [%s] is%s disabled.\n", pd->user,
+              locked ? "" : " not"));
+
+    if (locked) {
+        ret = pam_add_response(pd, SSS_PAM_SYSTEM_INFO,
+                               sizeof(NDS_DISABLE_MSG),
+                               (const uint8_t *) NDS_DISABLE_MSG);
+        if (ret != EOK) {
+            DEBUG(1, ("pam_add_response failed.\n"));
+        }
+    } else {
+        exp_time_str = ldb_msg_find_attr_as_string(user_entry,
+                                                SYSDB_NDS_LOGIN_EXPIRATION_TIME,
+                                                NULL);
+        locked = nds_check_expired(exp_time_str);
+
+        DEBUG(9, ("Account for user [%s] is%s expired.\n", pd->user,
+                  locked ? "" : " not"));
+
+        if (locked) {
+            ret = pam_add_response(pd, SSS_PAM_SYSTEM_INFO,
+                                   sizeof(NDS_EXPIRED_MSG),
+                                   (const uint8_t *) NDS_EXPIRED_MSG);
+            if (ret != EOK) {
+                DEBUG(1, ("pam_add_response failed.\n"));
+            }
+        } else {
+            time_map = ldb_msg_find_ldb_val(user_entry,
+                                            SYSDB_NDS_LOGIN_ALLOWED_TIME_MAP);
+
+            locked = nds_check_time_map(time_map);
+
+            DEBUG(9, ("Account for user [%s] is%s locked at this time.\n",
+                      pd->user, locked ? "" : " not"));
+
+            if (locked) {
+                ret = pam_add_response(pd, SSS_PAM_SYSTEM_INFO,
+                                       sizeof(NDS_TIME_MAP_MSG),
+                                       (const uint8_t *) NDS_TIME_MAP_MSG);
+                if (ret != EOK) {
+                    DEBUG(1, ("pam_add_response failed.\n"));
+                }
+            }
+        }
+    }
+
+    *pam_status = locked ? PAM_PERM_DENIED : PAM_SUCCESS;
+
+    return EOK;
+}
+
 struct sdap_account_expired_req_ctx {
     int pam_status;
 };
@@ -502,6 +696,12 @@ static struct tevent_req *sdap_account_expired_send(TALLOC_CTX *mem_ctx,
                 DEBUG(1, ("sdap_account_expired_rhds failed.\n"));
                 goto done;
             }
+        } else if (strcasecmp(expire, LDAP_ACCOUNT_EXPIRE_NDS) == 0) {
+            ret = sdap_account_expired_nds(pd, user_entry, &state->pam_status);
+            if (ret != EOK) {
+                DEBUG(1, ("sdap_account_expired_nds failed.\n"));
+                goto done;
+            }
         } else {
             DEBUG(1, ("Unsupported LDAP account expire policy [%s]. "
                       "Access denied.\n", expire));
@@ -1057,6 +1257,129 @@ static void sdap_access_service_done(struct tevent_req *subreq)
     return;
 }
 
+struct sdap_access_host_ctx {
+    int pam_status;
+};
+
+static struct tevent_req *sdap_access_host_send(
+        TALLOC_CTX *mem_ctx,
+        struct tevent_context *ev,
+        struct sdap_access_ctx *access_ctx,
+        struct pam_data *pd,
+        struct ldb_message *user_entry)
+{
+    errno_t ret;
+    struct tevent_req *req;
+    struct sdap_access_host_ctx *state;
+    struct ldb_message_element *el;
+    unsigned int i;
+    char *host;
+    char hostname[HOST_NAME_MAX+1];
+
+    req = tevent_req_create(mem_ctx, &state, struct sdap_access_host_ctx);
+    if (!req) {
+        return NULL;
+    }
+
+    state->pam_status = PAM_PERM_DENIED;
+
+    el = ldb_msg_find_element(user_entry, SYSDB_AUTHORIZED_HOST);
+    if (!el || el->num_values == 0) {
+        DEBUG(1, ("Missing hosts. Access denied\n"));
+        ret = EOK;
+        goto done;
+    }
+
+    if (gethostname(hostname, sizeof(hostname)) == -1) {
+        DEBUG(1, ("Unable to get system hostname. Access denied\n"));
+        ret = EOK;
+        goto done;
+    }
+
+    /* FIXME: PADL's pam_ldap also calls gethostbyname() on the hostname
+     *        in some attempt to get aliases and/or FQDN for the machine.
+     *        Not sure this is a good idea, but we might want to add it in
+     *        order to be compatible...
+     */
+
+    for (i = 0; i < el->num_values; i++) {
+        host = (char *)el->values[i].data;
+        if (host[0] == '!' &&
+                strcasecmp(hostname, host+1) == 0) {
+            /* This host is explicitly denied */
+            state->pam_status = PAM_PERM_DENIED;
+            DEBUG(4, ("Access denied by [%s]\n", host));
+            /* A denial trumps all. Break here */
+            break;
+
+        } else if (strcasecmp(hostname, host) == 0) {
+            /* This host is explicitly allowed */
+            state->pam_status = PAM_SUCCESS;
+            DEBUG(4, ("Access granted for [%s]\n", host));
+            /* We still need to loop through to make sure
+             * that it's not also explicitly denied
+             */
+        } else if (strcmp("*", host) == 0) {
+            /* This user has access to all hosts */
+            state->pam_status = PAM_SUCCESS;
+            DEBUG(4, ("Access granted to all hosts\n"));
+            /* We still need to loop through to make sure
+             * that it's not also explicitly denied
+             */
+        }
+    }
+
+    if (state->pam_status != PAM_SUCCESS) {
+        DEBUG(4, ("No matching host rule found\n"));
+    }
+
+    ret = EOK;
+
+done:
+    if (ret == EOK) {
+        tevent_req_done(req);
+    } else {
+        tevent_req_error(req, ret);
+    }
+    tevent_req_post(req, ev);
+
+    return req;
+}
+
+static errno_t sdap_access_host_recv(struct tevent_req *req,
+                                        int *pam_status)
+{
+    struct sdap_access_host_ctx *state =
+            tevent_req_data(req, struct sdap_access_host_ctx);
+
+    TEVENT_REQ_RETURN_ON_ERROR(req);
+
+    *pam_status = state->pam_status;
+
+    return EOK;
+}
+
+static void sdap_access_host_done(struct tevent_req *subreq)
+{
+    errno_t ret;
+    struct tevent_req *req = tevent_req_callback_data(subreq, struct tevent_req);
+    struct sdap_access_req_ctx *state =
+            tevent_req_data(req, struct sdap_access_req_ctx);
+
+    ret = sdap_access_host_recv(subreq, &state->pam_status);
+    talloc_zfree(subreq);
+    if (ret != EOK) {
+        DEBUG(1, ("Error retrieving access check result.\n"));
+        state->pam_status = PAM_SYSTEM_ERR;
+        tevent_req_error(req, ret);
+        return;
+    }
+
+    next_access_rule(req);
+
+    return;
+}
+
 static errno_t sdap_access_recv(struct tevent_req *req, int *pam_status)
 {
     struct sdap_access_req_ctx *state =
