@@ -498,6 +498,7 @@ static void sdap_id_op_connect_done(struct tevent_req *subreq)
                 tevent_req_callback_data(subreq, struct sdap_id_conn_data);
     struct sdap_id_conn_cache *conn_cache = conn_data->conn_cache;
     struct sdap_server_opts *srv_opts = NULL;
+    struct sdap_server_opts *current_srv_opts = NULL;
     bool can_retry = false;
     bool is_offline = false;
     int ret;
@@ -522,11 +523,25 @@ static void sdap_id_op_connect_done(struct tevent_req *subreq)
         /* be is going offline as there is no more servers to try */
         DEBUG(1, ("Failed to connect, going offline (%d [%s])\n",
                   ret, strerror(ret)));
-        be_mark_offline(conn_data->conn_cache->id_ctx->be);
+        be_mark_offline(conn_cache->id_ctx->be);
         is_offline = true;
     }
 
     if (ret == EOK) {
+        current_srv_opts = conn_cache->id_ctx->srv_opts;
+        if (current_srv_opts) {
+            DEBUG(8, ("Old USN: %lu, New USN: %lu\n", current_srv_opts->last_usn, srv_opts->last_usn));
+
+            if (strcmp(srv_opts->server_id, current_srv_opts->server_id) == 0 &&
+                srv_opts->supports_usn &&
+                current_srv_opts->last_usn > srv_opts->last_usn) {
+                DEBUG(5, ("Server was probably re-initialized\n"));
+
+                current_srv_opts->max_user_value = 0;
+                current_srv_opts->max_group_value = 0;
+                current_srv_opts->last_usn = srv_opts->last_usn;
+            }
+        }
         ret = sdap_id_conn_data_set_expire_timer(conn_data);
         sdap_steal_server_opts(conn_cache->id_ctx, &srv_opts);
     }
@@ -544,6 +559,8 @@ static void sdap_id_op_connect_done(struct tevent_req *subreq)
             default:
                 /* do not attempt to retry on errors like ENOMEM */
                 can_retry = false;
+                is_offline = true;
+                be_mark_offline(conn_cache->id_ctx->be);
                 break;
         }
     }
