@@ -54,7 +54,7 @@
 struct sbus_method monitor_nss_methods[] = {
     { MON_CLI_METHOD_PING, monitor_common_pong },
     { MON_CLI_METHOD_RES_INIT, monitor_common_res_init },
-    { MON_CLI_METHOD_ROTATE, monitor_common_rotate_logs },
+    { MON_CLI_METHOD_ROTATE, responder_logrotate },
     { NULL, NULL }
 };
 
@@ -157,7 +157,7 @@ static int nss_get_config(struct nss_ctx *nctx,
     if (ret != EOK) goto done;
 
     ret = confdb_get_int(cdb, nctx, CONFDB_NSS_CONF_ENTRY,
-                         CONFDB_NSS_ENTRY_CACHE_NOWAIT_PERCENTAGE, 0,
+                         CONFDB_NSS_ENTRY_CACHE_NOWAIT_PERCENTAGE, 50,
                          &nctx->cache_refresh_percent);
     if (ret != EOK) goto done;
     if (nctx->cache_refresh_percent < 0 ||
@@ -249,6 +249,17 @@ static void nss_dp_reconnect_init(struct sbus_connection *conn,
     /* nss_shutdown(rctx); */
 }
 
+static int nss_ctx_destructor(void *memctx)
+{
+    struct nss_ctx *nctx = (struct nss_ctx *) memctx;
+
+    if (nctx->netgroups) {
+        hash_destroy(nctx->netgroups);
+        nctx->netgroups = NULL;
+    }
+    return 0;
+}
+
 int nss_process_init(TALLOC_CTX *mem_ctx,
                      struct tevent_context *ev,
                      struct confdb_ctx *cdb)
@@ -264,6 +275,7 @@ int nss_process_init(TALLOC_CTX *mem_ctx,
         DEBUG(0, ("fatal error initializing nss_ctx\n"));
         return ENOMEM;
     }
+    talloc_set_destructor((TALLOC_CTX *) nctx, nss_ctx_destructor);
 
     ret = sss_ncache_init(nctx, &nctx->ncache);
     if (ret != EOK) {
