@@ -170,7 +170,7 @@ test_resolve_service_callback(struct tevent_req *req)
     if (task->new_server_status >= 0)
         fo_set_server_status(server, task->new_server_status);
 
-    if (strcmp(fo_get_server_name(server), "unknown name")) {
+    if (fo_get_server_name(server) != NULL) {
         he = fo_get_server_hostent(server);
         fail_if(he == NULL, "%s: fo_get_server_hostent() returned NULL");
         for (i = 0; he->addr_list[i]; i++) {
