@@ -19,7 +19,8 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#define _GNU_SOURCE
+#include "config.h"
+
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
@@ -31,14 +32,14 @@
 #include "util/util.h"
 
 const char *debug_prg_name = "sssd";
-int debug_level = 0;
+
+int debug_level = SSS_UNRESOLVED_DEBUG_LEVEL;
 int debug_timestamps = 1;
 
 int debug_to_file = 0;
 const char *debug_log_file = "sssd";
 FILE *debug_file = NULL;
 
-
 errno_t set_debug_file_from_fd(const int fd)
 {
     FILE *dummy;
