@@ -23,7 +23,6 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#define _GNU_SOURCE
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -31,6 +30,7 @@
 #include "util/util.h"
 #include "ldb.h"
 #include "confdb/confdb.h"
+#include "monitor/monitor_interfaces.h"
 
 #ifdef HAVE_PRCTL
 #include <sys/prctl.h>
@@ -346,6 +346,11 @@ int die_if_parent_died(void)
     return EOK;
 }
 
+struct logrotate_ctx {
+    struct confdb_ctx *confdb;
+    const char *confdb_path;
+};
+
 static void te_server_hup(struct tevent_context *ev,
                           struct tevent_signal *se,
                           int signum,
@@ -353,8 +358,17 @@ static void te_server_hup(struct tevent_context *ev,
                           void *siginfo,
                           void *private_data)
 {
+    errno_t ret;
+    struct logrotate_ctx *lctx =
+            talloc_get_type(private_data, struct logrotate_ctx);
+
     DEBUG(1, ("Received SIGHUP. Rotating logfiles.\n"));
-    rotate_debug_files();
+
+    ret = monitor_common_rotate_logs(lctx->confdb, lctx->confdb_path);
+    if (ret != EOK) {
+        DEBUG(0, ("Could not reopen log file [%s]\n",
+                  strerror(ret)));
+    }
 }
 
 int server_setup(const char *name, int flags,
@@ -369,6 +383,7 @@ int server_setup(const char *name, int flags,
     bool dt;
     bool dl;
     struct tevent_signal *tes;
+    struct logrotate_ctx *lctx;
 
     debug_prg_name = strdup(name);
     if (!debug_prg_name) {
@@ -444,14 +459,17 @@ int server_setup(const char *name, int flags,
         return ret;
     }
 
-    /* set debug level if any in conf_entry */
-    ret = confdb_get_int(ctx->confdb_ctx, ctx, conf_entry,
-                         CONFDB_SERVICE_DEBUG_LEVEL,
-                         debug_level, &debug_level);
-    if (ret != EOK) {
-        DEBUG(0, ("Error reading from confdb (%d) [%s]\n",
-                  ret, strerror(ret)));
-        return ret;
+    if (debug_level == SSS_UNRESOLVED_DEBUG_LEVEL) {
+        /* set debug level if any in conf_entry */
+        ret = confdb_get_int(ctx->confdb_ctx, ctx, conf_entry,
+                             CONFDB_SERVICE_DEBUG_LEVEL,
+                             SSS_DEFAULT_DEBUG_LEVEL,
+                             &debug_level);
+        if (ret != EOK) {
+            DEBUG(0, ("Error reading from confdb (%d) [%s]\n",
+                      ret, strerror(ret)));
+            return ret;
+        }
     }
 
     /* same for debug timestamps */
@@ -480,8 +498,14 @@ int server_setup(const char *name, int flags,
     if (dl) debug_to_file = 1;
 
     /* before opening the log file set up log rotation */
+    lctx = talloc_zero(ctx, struct logrotate_ctx);
+    if (!lctx) return ENOMEM;
+
+    lctx->confdb = ctx->confdb_ctx;
+    lctx->confdb_path = conf_entry;
+
     tes = tevent_add_signal(ctx->event_ctx, ctx, SIGHUP, 0,
-                            te_server_hup, NULL);
+                            te_server_hup, lctx);
     if (tes == NULL) {
         return EIO;
     }
