@@ -21,6 +21,7 @@
 #ifndef __SSSD_UTIL_H__
 #define __SSSD_UTIL_H__
 
+#include "config.h"
 #include <stdio.h>
 #include <stdint.h>
 #include <stdbool.h>
@@ -36,8 +37,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include "config.h"
-
 #include <talloc.h>
 #include <tevent.h>
 #include <ldb.h>
@@ -60,6 +59,9 @@ extern const char *debug_log_file;
 void debug_fn(const char *format, ...);
 errno_t set_debug_file_from_fd(const int fd);
 
+#define SSS_DEFAULT_DEBUG_LEVEL 0
+#define SSS_UNRESOLVED_DEBUG_LEVEL -1
+
 #define SSSD_DEBUG_OPTS \
         {"debug-level", 'd', POPT_ARG_INT, &debug_level, 0, \
          _("Debug level"), NULL}, \
@@ -403,4 +405,7 @@ errno_t sss_filter_sanitize(TALLOC_CTX *mem_ctx,
                             const char *input,
                             char **sanitized);
 
+char *
+sss_escape_ip_address(TALLOC_CTX *mem_ctx, int family, const char *addr);
+
 #endif /* __SSSD_UTIL_H__ */
